@@ -1,11 +1,17 @@
 getData <- function(filename=NULL,
                     errorMessage = "[defaultErrorMessage]",
-                    use.value.labels=TRUE,
+                    use.value.labels=FALSE,
                     to.data.frame=TRUE,
-                    stringsAsFactors=TRUE, ...) {
+                    stringsAsFactors=FALSE, ...) {
+  
+  dots <- list(...);
+  fullArguments <- as.list(environment());
+  matchedCall <- match.call();
+  fullCall <- capture.output(print(matchedCall));
+  filenameArgument <- filename;
   
   ### File formats that have been implemented
-  supportedFormats <- c(".sav", ".csv", ".tsv", ".ods", ".xls", "xlsx");
+  supportedFormats <- c(".sav", ".csv", ".tsv", ".rda", ".ods", ".xls", "xlsx");
   
   ### Set error message
   errorMessage <- sub("\\[defaultErrorMessage\\]",
@@ -16,9 +22,35 @@ getData <- function(filename=NULL,
   
   if (is.null(filename)) {
     ### If no filename is specified, request one from the user
+    cat("You did not specify a file to open. Therefore, please select the",
+        "file to open in the File Selection Dialog.",
+        "Note that this dialog can sometimes appear behind the R window.",
+        "If you do not see the file dialog now, use ALT-TAB or check the ",
+        "start bar (in Windows), use COMMAND-TAB (in OSX), or check the ",
+        "dock (in *nux based systems such as",
+        "Ubuntu or OS X).");
     filename <- file.choose();
+    slashesFilename <- gsub("\\", "/", filename, fixed=TRUE);
+    
+    if (length(matchedCall) == 1) {
+      filenameArgument <- sub('getData(', paste0('getData(filename="',
+                                                 trim(slashesFilename), '"'),
+                              fullCall, fixed=TRUE);
+    } else {
+      filenameArgument <- sub('getData(', paste0('getData(filename="',
+                                                 trim(slashesFilename), '", '),
+                              fullCall, fixed=TRUE);
+    }
+    
+    filenameArgument <- gsub(", ", ",\n        ", filenameArgument, fixed=TRUE);
+    
+    cat("\n\nYou have selected a file. Based on your call and the filename",
+        "and directory (path) you selected, this is the",
+        "command you can use to read the datafile without",
+        "a dialog, for example in an R script:\n\n");
+    cat(filenameArgument, ";\n\n", sep="");
   }
-
+  
   extension <- tolower(substring(filename, nchar(filename) - 3));
 
   if (!file.exists(filename) |
@@ -27,9 +59,14 @@ getData <- function(filename=NULL,
     stop(errorMessage);
   }
   else {
+    if (extension == ".rda") {
+      dat <- load(filename);
+      dat <- get(dat);
+    }
     if (extension == ".sav") {
       dat <- suppressWarnings(read.spss(filename, use.value.labels=use.value.labels,
                               to.data.frame=to.data.frame, ...));
+      
 #       dat <- read.spss(filename, use.value.labels=use.value.labels,
 #                        to.data.frame=to.data.frame, ...);
 #       cat("Note that a warning like 'Unrecognized record type 7, subtype ## encountered in system file'",
@@ -75,26 +112,127 @@ getData <- function(filename=NULL,
     }
     else if ((extension == ".xls") || (extension == "xlsx")) {
       if (!is.element('XLConnect', installed.packages()[, 1])) {
-         stop("To load Excel (.xls or .xlsx) files, I need package 'XLConnect', ",
-              "which in turn requires Java. Please install it yourself if you wish to ",
-              "use this. You can insstall is using:\n\n",
-              "install.packages('XLConnect')\n\nOf course, you can always export from ",
-              "Excel to .csv (comma separated values) and load that file.");
+        stop("To load Excel (.xls or .xlsx) files, I need package 'XLConnect', ",
+             "which in turn requires Java. Please install it yourself if you wish to ",
+             "use this. You can install it using:\n\n",
+             "install.packages('XLConnect')\n\nOf course, you can always export from ",
+             "Excel to .csv (comma separated values) and load that file.");
       }
       else {
-        require('XLConnect');
-        wb <- loadWorkbook(filename, ...);
-        dat <- readWorksheet(wb, sheet=1);
-      }
-    }
+        wb <- XLConnect::loadWorkbook(filename, ...);
+        dat <- XLConnect::readWorksheet(wb, sheet=1);
+        if (requireNamespace('XLConnect')) {
+          wb <- XLConnect::loadWorkbook(filename, ...);
+          dat <- XLConnect::readWorksheet(wb, sheet=1);
+        } else {
+          stop("To load Excel (.xls or .xlsx) files, I need package 'XLConnect', ",
+               "which in turn requires Java. Please install it yourself if you wish to ",
+               "use this. You can install it using:\n\n",
+               "install.packages('XLConnect')\n\nOf course, you can always export from ",
+               "Excel to .csv (comma separated values) and load that file.");
+        }
+     }
+   }
     
     ### Store the file where we got this dataframe
     attr(dat, "fileName") <- filename;
+    ### Store the call
+    attr(dat, "getDataCall") <- filenameArgument;
+    
     ### Return the resuls
     return(dat);
   }  
 }
 
+getDat <- function(dfName="dat", backup=TRUE, ...) {
+  dat <- getData(...);
+  if (exists(dfName, envir=sys.frame(-1)) && backup) {
+    backupName <- paste0(dfName, '_backup_',
+                         format(Sys.time(), "%Y%m%d_%H%M%S"));
+    assign(backupName,
+           value=get(dfName, envir=sys.frame(-1)),
+           envir=sys.frame(-1));
+    warning("NOTE: an object called '", dfName, "' already existed; I renamed ",
+            "it to '", backupName, "'.");
+  }
+  assign(dfName, value=dat, envir=sys.frame(-1));
+  cat("The data has been stored in a dataframe called '",
+      dfName, "'. That means that if you want to repeat this command and ",
+      "store the dataframe with the same name, you have to use:\n\n",
+      dfName, " <- ", attributes(dat)$getDataCall, ";\n\n",      
+      sep="");
+}
+
+
+exportToSPSS <- function (dat, datafile, codefile, fileEncoding = "UTF-8",
+                          newLinesInString = " |n| ") {
+  
+  ### Convert newline characters to spaces
+  if (any(charVectors <- sapply(dat, is.character))) {
+    dat[, charVectors] <- data.frame(lapply(dat[, charVectors],
+                                           function(x) {
+                                             return(gsub('\n', newLinesInString,
+                                                         x));
+                                           }), stringsAsFactors=FALSE);
+  }
+  
+  ### Export datafile
+  write.table(massConvertToNumeric(dat), file = datafile,
+              row.names = FALSE, col.names = TRUE, 
+              sep = ",", quote = TRUE, na = "",
+              fileEncoding = fileEncoding);
+
+  codeFileConnection=file(codefile, open="w", encoding=fileEncoding);
+  
+  cat(paste0("GET DATA
+  /TYPE = TXT
+  /FILE = \"", datafile, "\"
+  /DELIMITERS = \",\"
+  /QUALIFIER = '\"'
+  /FIRSTCASE = 2
+  /VARIABLES =\n"), file=codeFileConnection);
+  
+  varlabels = names(dat);
+  varnames = gsub("[^[:alnum:]_\\$@#]", "\\.", names(dat));
+  
+  cat(paste0("  ", varnames, " ",
+             unlist(lapply(dat, function(x) {
+               if (is.character(x)) {
+                 return(paste0('A', max(nchar(x))));
+               } else {
+                 return("F8.2");
+               }
+             })), collapse="\n"), file=codeFileConnection, append=TRUE);
+
+  cat(".\n\nVARIABLE LABELS\n", file = codeFileConnection, append = TRUE);
+  
+  cat(paste(varnames,
+            paste("\"", varlabels, "\"", sep = ""),
+            "\n"), ".\n", file = codeFileConnection, 
+      append = TRUE);
+  
+  factors <- sapply(dat, is.factor);
+  
+  if (any(factors)) {
+    cat("\nVALUE LABELS\n", file = codeFileConnection, append = TRUE);
+    for (v in which(factors)) {
+      cat("/\n", file = codeFileConnection, append = TRUE);
+      cat(varnames[v], " \n", file = codeFileConnection, append = TRUE, 
+          sep = "");
+      levs <- levels(dat[[v]]);
+      cat(paste(seq_along(levs),
+                paste("\"", levs, "\"", sep = ""),
+                "\n", sep = " "), 
+          file = codeFileConnection, append = TRUE);
+    }
+    cat(".\n", file = codeFileConnection, append = TRUE);
+  }
+  
+  cat("\nEXECUTE.\n", file = codeFileConnection, append = TRUE);
+  
+  close(codeFileConnection);
+}
+
 mediaan <- function(vector) {
   if (is.data.frame(vector) | is.matrix(vector)) {
     stop("The first argument is not a vector! If you need to specify ",
